@@ -1,4 +1,4 @@
-use std::{char::decode_utf16, sync::Arc};
+use std::{char::decode_utf16, convert::TryInto, sync::Arc};
 
 use arrow::array::{ArrayRef, StringBuilder};
 use odbc_api::{
@@ -6,17 +6,17 @@ use odbc_api::{
     DataType as OdbcDataType,
 };
 
-use crate::Error;
-
-use super::ColumnStrategy;
+use super::{ColumnFailure, ColumnStrategy};
 
+/// This function decides wether this column will be queried as narrow (assumed to be utf-8) or
+/// wide text (assumed to be utf-16). The reason we do not always use narrow is that the encoding
+/// dependends on the system locals which is usually not UTF-8 on windows systems. Furthermore we
+/// are trying to adapt the buffer size to the maximum string length the column could contain.
 pub fn choose_text_strategy(
-    lazy_sql_type: impl Fn() -> Result<OdbcDataType, Error>,
-    lazy_octet_size: impl Fn() -> Result<isize, odbc_api::Error>,
+    sql_type: OdbcDataType,
     lazy_display_size: impl Fn() -> Result<isize, odbc_api::Error>,
     is_nullable: bool,
-) -> Result<Box<dyn ColumnStrategy>, Error> {
-    let sql_type = lazy_sql_type()?;
+) -> Result<Box<dyn ColumnStrategy>, ColumnFailure> {
     let is_narrow = matches!(
         sql_type,
         OdbcDataType::LongVarchar { .. } | OdbcDataType::Varchar { .. } | OdbcDataType::Char { .. }
@@ -27,66 +27,43 @@ pub fn choose_text_strategy(
     );
     let is_text = is_narrow || is_wide;
     Ok(if is_text {
-        let octet_len =
-            lazy_octet_size().map_err(|source| Error::UnknownStringLength { sql_type, source })?;
         if cfg!(target_os = "windows") {
-            let octet_len = if is_narrow {
-                // Narrow column queried from a windows platform => Utf8 to Utf16
-                // conversion. The letters in the range from U+0000 to U+007F take two bytes
-                // in UTF-16 but only one byte in UTF-8
-                octet_len * 2
-            } else {
-                // Wide text column on database queried from a windows platform.
-                octet_len
-            };
-            // Use wide text in windows as default locale can not be expected to be UTF-8
-            wide_text_strategy(octet_len, is_nullable)?
+            let hex_len = sql_type.utf16_len().unwrap();
+            if hex_len == 0 {
+                return Err(ColumnFailure::ZeroSizedColumn { sql_type });
+            }
+            wide_text_strategy(hex_len, is_nullable)
         } else {
-            let octet_len = if is_wide {
-                // Wide column queried on non-window platform => Utf16 to Utf8 conversion.
-                // We must adjust the binary size reported by the database, since it assumes
-                // UTF-16 encoding. While usually an UTF-8 string is shorter for characters
-                // in the range from U+0800 to U+FFFF UTF-8 is three bytes theras UTF-16 is
-                // only two bytes large. We have to allocate enough memory to hold the
-                // largest possible string.
-                (octet_len / 2) * 3
-            } else {
-                // Narrow column queried on non-windows platform => Utf8 to Utf8
-                octet_len
-            };
-            narrow_text_strategy(octet_len, is_nullable)?
+            let octet_len = sql_type.utf8_len().unwrap();
+            if octet_len == 0 {
+                return Err(ColumnFailure::ZeroSizedColumn { sql_type });
+            }
+            narrow_text_strategy(octet_len, is_nullable)
         }
     } else {
-        let display_size = lazy_display_size()
-            .map_err(|source| Error::UnknownStringLength { sql_type, source })?;
+        let display_size: usize = sql_type
+            .display_size()
+            .map(|ds| Ok(ds as isize))
+            .unwrap_or_else(lazy_display_size)
+            .map_err(|source| ColumnFailure::UnknownStringLength { sql_type, source })?
+            .try_into()
+            .unwrap();
+
+        if display_size == 0 {
+            return Err(ColumnFailure::ZeroSizedColumn { sql_type });
+        }
+
         // We assume non text type colmuns to only consist of ASCII characters.
-        narrow_text_strategy(display_size, is_nullable)?
+        narrow_text_strategy(display_size, is_nullable)
     })
 }
 
-fn wide_text_strategy(
-    octet_length: isize,
-    is_nullable: bool,
-) -> Result<Box<dyn ColumnStrategy>, Error> {
-    if octet_length < 1 {
-        return Err(Error::InvalidDisplaySize(octet_length));
-    }
-    let octet_length = octet_length as usize;
-    // An octet is a byte, a u16 consists of two bytes therefore we are dividing by
-    // two to get the correct length.
-    let utf16_len = octet_length / 2;
-    Ok(Box::new(WideText::new(is_nullable, utf16_len)))
+fn wide_text_strategy(u16_len: usize, is_nullable: bool) -> Box<dyn ColumnStrategy> {
+    Box::new(WideText::new(is_nullable, u16_len))
 }
 
-fn narrow_text_strategy(
-    octet_len: isize,
-    is_nullable: bool,
-) -> Result<Box<dyn ColumnStrategy>, Error> {
-    if octet_len < 1 {
-        return Err(Error::InvalidDisplaySize(octet_len));
-    }
-    let utf8_len = octet_len as usize;
-    Ok(Box::new(NarrowText::new(is_nullable, utf8_len)))
+fn narrow_text_strategy(octet_len: usize, is_nullable: bool) -> Box<dyn ColumnStrategy> {
+    Box::new(NarrowText::new(is_nullable, octet_len))
 }
 
 /// Strategy requesting the text from the database as UTF-16 (Wide characters) and emmitting it as
